@@ -1,6 +1,7 @@
-/* $OpenBSD: comp_err.c,v 1.9 2014/11/03 16:58:28 tedu Exp $ */
+/* $OpenBSD: comp_err.c,v 1.13 2022/12/23 23:23:02 schwarze Exp $ */
 /* ====================================================================
- * Copyright (c) 1999-2007 The OpenSSL Project.  All rights reserved.
+ * This file was written by Dr. Stephen Henson <steve@openssl.org>.
+ * Copyright (c) 1999, 2005, 2008 The OpenSSL Project.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -60,7 +61,8 @@
 #include <openssl/comp.h>
 #include <openssl/err.h>
 
-/* BEGIN ERROR CODES */
+#include "comp_local.h"
+
 #ifndef OPENSSL_NO_ERR
 
 #define ERR_FUNC(func) ERR_PACK(ERR_LIB_COMP,func,0)
