@@ -1,4 +1,4 @@
-/* $OpenBSD: ecp_oct.c,v 1.12 2020/12/04 08:55:30 tb Exp $ */
+/* $OpenBSD: ecp_oct.c,v 1.19 2022/11/26 16:08:52 tb Exp $ */
 /* Includes code written by Lenka Fibikova <fibikova@exp-math.uni-essen.de>
  * for the OpenSSL project.
  * Includes code written by Bodo Moeller for the OpenSSL project.
@@ -64,11 +64,11 @@
 
 #include <openssl/err.h>
 
-#include "ec_lcl.h"
+#include "ec_local.h"
 
-int 
-ec_GFp_simple_set_compressed_coordinates(const EC_GROUP * group,
-    EC_POINT * point, const BIGNUM * x_, int y_bit, BN_CTX * ctx)
+int
+ec_GFp_simple_set_compressed_coordinates(const EC_GROUP *group,
+    EC_POINT *point, const BIGNUM *x_, int y_bit, BN_CTX *ctx)
 {
 	BN_CTX *new_ctx = NULL;
 	BIGNUM *tmp1, *tmp2, *x, *y;
@@ -162,30 +162,17 @@ ec_GFp_simple_set_compressed_coordinates(const EC_GROUP * group,
 	}
 	if (y_bit != BN_is_odd(y)) {
 		if (BN_is_zero(y)) {
-			int kron;
-
-			kron = BN_kronecker(x, &group->field, ctx);
-			if (kron == -2)
-				goto err;
-
-			if (kron == 1)
-				ECerror(EC_R_INVALID_COMPRESSION_BIT);
-			else
-				/*
-				 * BN_mod_sqrt() should have cought this
-				 * error (not a square)
-				 */
-				ECerror(EC_R_INVALID_COMPRESSED_POINT);
+			ECerror(EC_R_INVALID_COMPRESSION_BIT);
 			goto err;
 		}
 		if (!BN_usub(y, &group->field, y))
 			goto err;
+		if (y_bit != BN_is_odd(y)) {
+			ECerror(ERR_R_INTERNAL_ERROR);
+			goto err;
+		}
 	}
-	if (y_bit != BN_is_odd(y)) {
-		ECerror(ERR_R_INTERNAL_ERROR);
-		goto err;
-	}
-	if (!EC_POINT_set_affine_coordinates_GFp(group, point, x, y, ctx))
+	if (!EC_POINT_set_affine_coordinates(group, point, x, y, ctx))
 		goto err;
 
 	ret = 1;
@@ -197,9 +184,9 @@ ec_GFp_simple_set_compressed_coordinates(const EC_GROUP * group,
 }
 
 
-size_t 
-ec_GFp_simple_point2oct(const EC_GROUP * group, const EC_POINT * point, point_conversion_form_t form,
-    unsigned char *buf, size_t len, BN_CTX * ctx)
+size_t
+ec_GFp_simple_point2oct(const EC_GROUP *group, const EC_POINT *point, point_conversion_form_t form,
+    unsigned char *buf, size_t len, BN_CTX *ctx)
 {
 	size_t ret;
 	BN_CTX *new_ctx = NULL;
@@ -246,7 +233,7 @@ ec_GFp_simple_point2oct(const EC_GROUP * group, const EC_POINT * point, point_co
 		if ((y = BN_CTX_get(ctx)) == NULL)
 			goto err;
 
-		if (!EC_POINT_get_affine_coordinates_GFp(group, point, x, y, ctx))
+		if (!EC_POINT_get_affine_coordinates(group, point, x, y, ctx))
 			goto err;
 
 		if ((form == POINT_CONVERSION_COMPRESSED || form == POINT_CONVERSION_HYBRID) && BN_is_odd(y))
@@ -302,9 +289,9 @@ ec_GFp_simple_point2oct(const EC_GROUP * group, const EC_POINT * point, point_co
 }
 
 
-int 
-ec_GFp_simple_oct2point(const EC_GROUP * group, EC_POINT * point,
-    const unsigned char *buf, size_t len, BN_CTX * ctx)
+int
+ec_GFp_simple_oct2point(const EC_GROUP *group, EC_POINT *point,
+    const unsigned char *buf, size_t len, BN_CTX *ctx)
 {
 	point_conversion_form_t form;
 	int y_bit;
@@ -363,10 +350,10 @@ ec_GFp_simple_oct2point(const EC_GROUP * group, EC_POINT * point,
 	}
 	if (form == POINT_CONVERSION_COMPRESSED) {
 		/*
-		 * EC_POINT_set_compressed_coordinates_GFp checks that the point
+		 * EC_POINT_set_compressed_coordinates checks that the point
 		 * is on the curve as required by X9.62.
 		 */
-		if (!EC_POINT_set_compressed_coordinates_GFp(group, point, x, y_bit, ctx))
+		if (!EC_POINT_set_compressed_coordinates(group, point, x, y_bit, ctx))
 			goto err;
 	} else {
 		if (!BN_bin2bn(buf + 1 + field_len, field_len, y))
@@ -382,10 +369,10 @@ ec_GFp_simple_oct2point(const EC_GROUP * group, EC_POINT * point,
 			}
 		}
 		/*
-		 * EC_POINT_set_affine_coordinates_GFp checks that the point is
+		 * EC_POINT_set_affine_coordinates checks that the point is
 		 * on the curve as required by X9.62.
 		 */
-		if (!EC_POINT_set_affine_coordinates_GFp(group, point, x, y, ctx))
+		if (!EC_POINT_set_affine_coordinates(group, point, x, y, ctx))
 			goto err;
 	}
 
